@@ -1,4 +1,4 @@
-/*	$OpenBSD: cert.c,v 1.43 2021/10/28 09:02:19 beck Exp $ */
+/*	$OpenBSD: cert.c,v 1.47 2021/11/05 10:50:41 claudio Exp $ */
 /*
  * Copyright (c) 2021 Job Snijders <job@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -1084,11 +1084,18 @@ cert_parse_inner(X509 **xp, const char *fn, const unsigned char *der,
 			goto out;
 		}
 		if (p.res->ipsz > 0) {
-			warnx("%s: unexpected IP resources in BGPsec cert", p.fn);
+			warnx("%s: unexpected IP resources in BGPsec cert",
+			   p.fn);
 			goto out;
 		}
 		if (sia_present) {
-			warnx("%s: unexpected SIA extension in BGPsec cert", p.fn);
+			warnx("%s: unexpected SIA extension in BGPsec cert",
+			   p.fn);
+			goto out;
+		}
+		if (ta) {
+			warnx("%s: BGPsec cert can not be a trust anchor",
+			   p.fn);
 			goto out;
 		}
 		break;
@@ -1213,40 +1220,11 @@ cert_free(struct cert *p)
 	free(p->aia);
 	free(p->aki);
 	free(p->ski);
-	free(p->tal);
 	free(p->pubkey);
 	X509_free(p->x509);
 	free(p);
 }
 
-static void
-cert_ip_buffer(struct ibuf *b, const struct cert_ip *p)
-{
-	io_simple_buffer(b, &p->afi, sizeof(enum afi));
-	io_simple_buffer(b, &p->type, sizeof(enum cert_ip_type));
-
-	if (p->type != CERT_IP_INHERIT) {
-		io_simple_buffer(b, &p->min, sizeof(p->min));
-		io_simple_buffer(b, &p->max, sizeof(p->max));
-	}
-
-	if (p->type == CERT_IP_RANGE)
-		ip_addr_range_buffer(b, &p->range);
-	else if (p->type == CERT_IP_ADDR)
-		ip_addr_buffer(b, &p->ip);
-}
-
-static void
-cert_as_buffer(struct ibuf *b, const struct cert_as *p)
-{
-	io_simple_buffer(b, &p->type, sizeof(enum cert_as_type));
-	if (p->type == CERT_AS_RANGE) {
-		io_simple_buffer(b, &p->range.min, sizeof(uint32_t));
-		io_simple_buffer(b, &p->range.max, sizeof(uint32_t));
-	} else if (p->type == CERT_AS_ID)
-		io_simple_buffer(b, &p->id, sizeof(uint32_t));
-}
-
 /*
  * Write certificate parsed content into buffer.
  * See cert_read() for the other side of the pipe.
@@ -1254,18 +1232,15 @@ cert_as_buffer(struct ibuf *b, const struct cert_as *p)
 void
 cert_buffer(struct ibuf *b, const struct cert *p)
 {
-	size_t	 i;
+	io_simple_buffer(b, &p->expires, sizeof(p->expires));
+	io_simple_buffer(b, &p->purpose, sizeof(p->purpose));
+	io_simple_buffer(b, &p->talid, sizeof(p->talid));
+	io_simple_buffer(b, &p->ipsz, sizeof(p->ipsz));
+	io_simple_buffer(b, &p->asz, sizeof(p->asz));
 
-	io_simple_buffer(b, &p->valid, sizeof(int));
-	io_simple_buffer(b, &p->expires, sizeof(time_t));
-	io_simple_buffer(b, &p->purpose, sizeof(enum cert_purpose));
-	io_simple_buffer(b, &p->ipsz, sizeof(size_t));
-	for (i = 0; i < p->ipsz; i++)
-		cert_ip_buffer(b, &p->ips[i]);
+	io_simple_buffer(b, p->ips, p->ipsz * sizeof(p->ips[0]));
+	io_simple_buffer(b, p->as, p->asz * sizeof(p->as[0]));
 
-	io_simple_buffer(b, &p->asz, sizeof(size_t));
-	for (i = 0; i < p->asz; i++)
-		cert_as_buffer(b, &p->as[i]);
 	io_str_buffer(b, p->mft);
 	io_str_buffer(b, p->notify);
 	io_str_buffer(b, p->repo);
@@ -1273,38 +1248,9 @@ cert_buffer(struct ibuf *b, const struct cert *p)
 	io_str_buffer(b, p->aia);
 	io_str_buffer(b, p->aki);
 	io_str_buffer(b, p->ski);
-	io_str_buffer(b, p->tal);
 	io_str_buffer(b, p->pubkey);
 }
 
-static void
-cert_ip_read(struct ibuf *b, struct cert_ip *p)
-{
-	io_read_buf(b, &p->afi, sizeof(enum afi));
-	io_read_buf(b, &p->type, sizeof(enum cert_ip_type));
-
-	if (p->type != CERT_IP_INHERIT) {
-		io_read_buf(b, &p->min, sizeof(p->min));
-		io_read_buf(b, &p->max, sizeof(p->max));
-	}
-
-	if (p->type == CERT_IP_RANGE)
-		ip_addr_range_read(b, &p->range);
-	else if (p->type == CERT_IP_ADDR)
-		ip_addr_read(b, &p->ip);
-}
-
-static void
-cert_as_read(struct ibuf *b, struct cert_as *p)
-{
-	io_read_buf(b, &p->type, sizeof(enum cert_as_type));
-	if (p->type == CERT_AS_RANGE) {
-		io_read_buf(b, &p->range.min, sizeof(uint32_t));
-		io_read_buf(b, &p->range.max, sizeof(uint32_t));
-	} else if (p->type == CERT_AS_ID)
-		io_read_buf(b, &p->id, sizeof(uint32_t));
-}
-
 /*
  * Allocate and read parsed certificate content from descriptor.
  * The pointer must be freed with cert_free().
@@ -1314,28 +1260,25 @@ struct cert *
 cert_read(struct ibuf *b)
 {
 	struct cert	*p;
-	size_t		 i;
 
 	if ((p = calloc(1, sizeof(struct cert))) == NULL)
 		err(1, NULL);
 
-	io_read_buf(b, &p->valid, sizeof(int));
-	io_read_buf(b, &p->expires, sizeof(time_t));
-	io_read_buf(b, &p->purpose, sizeof(enum cert_purpose));
-	io_read_buf(b, &p->ipsz, sizeof(size_t));
+	io_read_buf(b, &p->expires, sizeof(p->expires));
+	io_read_buf(b, &p->purpose, sizeof(p->purpose));
+	io_read_buf(b, &p->talid, sizeof(p->talid));
+	io_read_buf(b, &p->ipsz, sizeof(p->ipsz));
+	io_read_buf(b, &p->asz, sizeof(p->asz));
 
 	p->ips = calloc(p->ipsz, sizeof(struct cert_ip));
 	if (p->ips == NULL)
 		err(1, NULL);
-	for (i = 0; i < p->ipsz; i++)
-		cert_ip_read(b, &p->ips[i]);
+	io_read_buf(b, p->ips, p->ipsz * sizeof(p->ips[0]));
 
-	io_read_buf(b, &p->asz, sizeof(size_t));
 	p->as = calloc(p->asz, sizeof(struct cert_as));
 	if (p->as == NULL)
 		err(1, NULL);
-	for (i = 0; i < p->asz; i++)
-		cert_as_read(b, &p->as[i]);
+	io_read_buf(b, p->as, p->asz * sizeof(p->as[0]));
 
 	io_read_str(b, &p->mft);
 	io_read_str(b, &p->notify);
@@ -1344,7 +1287,6 @@ cert_read(struct ibuf *b)
 	io_read_str(b, &p->aia);
 	io_read_str(b, &p->aki);
 	io_read_str(b, &p->ski);
-	io_read_str(b, &p->tal);
 	io_read_str(b, &p->pubkey);
 
 	assert(p->mft != NULL || p->purpose == CERT_PURPOSE_BGPSEC_ROUTER);
@@ -1365,6 +1307,24 @@ auth_find(struct auth_tree *auths, const char *aki)
 	return RB_FIND(auth_tree, auths, &a);
 }
 
+int
+auth_insert(struct auth_tree *auths, struct cert *cert, struct auth *parent)
+{
+	struct auth *na;
+
+	na = malloc(sizeof(*na));
+	if (na == NULL)
+		err(1, NULL);
+
+	na->parent = parent;
+	na->cert = cert;
+
+	if (RB_INSERT(auth_tree, auths, na) != NULL)
+		err(1, "auth tree corrupted");
+
+	return 1;
+}
+
 static inline int
 authcmp(struct auth *a, struct auth *b)
 {
@@ -1383,8 +1343,7 @@ insert_brk(struct brk_tree *tree, struct cert *cert, int asid)
 
 	b->asid = asid;
 	b->expires = cert->expires;
-	if ((b->tal = strdup(cert->tal)) == NULL)
-		err(1, NULL);
+	b->talid = cert->talid;
 	if ((b->ski = strdup(cert->ski)) == NULL)
 		err(1, NULL);
 	if ((b->pubkey = strdup(cert->pubkey)) == NULL)
@@ -1397,13 +1356,10 @@ insert_brk(struct brk_tree *tree, struct cert *cert, int asid)
 	if ((found = RB_INSERT(brk_tree, tree, b)) != NULL) {
 		if (found->expires < b->expires) {
 			found->expires = b->expires;
-			free(found->tal);
-			found->tal = b->tal;
-			b->tal = NULL;
+			found->talid = b->talid;
 		}
 		free(b->ski);
 		free(b->pubkey);
-		free(b->tal);
 		free(b);
 	}
 }
