@@ -1,4 +1,4 @@
-/*	$OpenBSD: extern.h,v 1.86 2021/10/29 09:27:36 claudio Exp $ */
+/*	$OpenBSD: extern.h,v 1.95 2021/11/09 11:03:39 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -118,6 +118,7 @@ struct cert {
 	size_t		 ipsz; /* length of "ips" */
 	struct cert_as	*as; /* list of AS numbers and ranges */
 	size_t		 asz; /* length of "asz" */
+	int		 talid; /* cert is covered by which TAL */
 	char		*repo; /* CA repository (rsync:// uri) */
 	char		*mft; /* manifest (rsync:// uri) */
 	char		*notify; /* RRDP notify (https:// uri) */
@@ -125,10 +126,8 @@ struct cert {
 	char		*aia; /* AIA (or NULL, for trust anchor) */
 	char		*aki; /* AKI (or NULL, for trust anchor) */
 	char		*ski; /* SKI */
-	char		*tal; /* basename of TAL for this cert */
-	enum cert_purpose	 purpose; /* Certificate Purpose (BGPSec or CA) */
+	enum cert_purpose	 purpose; /* BGPSec or CA */
 	char		*pubkey; /* Subject Public Key Info */
-	int		 valid; /* validated resources */
 	X509		*x509; /* the cert */
 	time_t		 expires; /* do not use after */
 };
@@ -146,6 +145,7 @@ struct tal {
 	unsigned char	*pkey; /* DER-encoded public key */
 	size_t		 pkeysz; /* length of pkey */
 	char		*descr; /* basename of tal file */
+	int		 id; /* ID of this TAL */
 };
 
 /*
@@ -193,11 +193,11 @@ struct roa {
 	uint32_t	 asid; /* asID of ROA (if 0, RFC 6483 sec 4) */
 	struct roa_ip	*ips; /* IP prefixes */
 	size_t		 ipsz; /* number of IP prefixes */
+	int		 talid; /* ROAs are covered by which TAL */
 	int		 valid; /* validated resources */
 	char		*aia; /* AIA */
 	char		*aki; /* AKI */
 	char		*ski; /* SKI */
-	char		*tal; /* basename of TAL for this cert */
 	time_t		 expires; /* do not use after */
 };
 
@@ -217,8 +217,8 @@ struct gbr {
 struct vrp {
 	RB_ENTRY(vrp)	entry;
 	struct ip_addr	addr;
+	int		talid; /* covered by which TAL */
 	uint32_t	asid;
-	char		*tal; /* basename of TAL for this cert */
 	enum afi	afi;
 	unsigned char	maxlength;
 	time_t		expires; /* transitive expiry moment */
@@ -235,7 +235,7 @@ RB_PROTOTYPE(vrp_tree, vrp, entry, vrpcmp);
 struct brk {
 	RB_ENTRY(brk)	 entry;
 	uint32_t	 asid;
-	char		*tal; /* basename of TAL for this key */
+	int		 talid; /* covered by which TAL */
 	char		*ski; /* Subject Key Identifier */
 	char		*pubkey; /* Subject Public Key Info */
 	time_t		 expires; /* transitive expiry moment */
@@ -270,8 +270,6 @@ struct auth {
 	RB_ENTRY(auth)	 entry;
 	struct cert	*cert; /* owner information */
 	struct auth	*parent; /* pointer to parent or NULL for TA cert */
-	char		*tal; /* basename of TAL for this cert */
-	char		*fn; /* FIXME: debugging */
 };
 /*
  * Tree of auth sorted by ski
@@ -279,7 +277,8 @@ struct auth {
 RB_HEAD(auth_tree, auth);
 RB_PROTOTYPE(auth_tree, auth, entry, authcmp);
 
-struct auth *auth_find(struct auth_tree *, const char *);
+struct auth	*auth_find(struct auth_tree *, const char *);
+int		 auth_insert(struct auth_tree *, struct cert *, struct auth *);
 
 /*
  * Resource types specified by the RPKI profiles.
@@ -342,7 +341,7 @@ struct entity {
 	int		 has_data;	/* whether data blob is specified */
 	unsigned char	*data;		/* optional data blob */
 	size_t		 datasz; 	/* length of optional data blob */
-	char		*descr;		/* tal description */
+	int		 talid;		/* tal identifier */
 	TAILQ_ENTRY(entity) entries;
 };
 TAILQ_HEAD(entityq, entity);
@@ -361,8 +360,7 @@ struct stats {
 	size_t	 mfts_fail; /* failing syntactic parse */
 	size_t	 mfts_stale; /* stale manifests */
 	size_t	 certs; /* certificates */
-	size_t	 certs_fail; /* failing syntactic parse */
-	size_t	 certs_invalid; /* invalid resources */
+	size_t	 certs_fail; /* invalid certificate */
 	size_t	 roas; /* route origin authorizations */
 	size_t	 roas_fail; /* failing syntactic parse */
 	size_t	 roas_invalid; /* invalid resources */
@@ -380,8 +378,6 @@ struct stats {
 	size_t	 del_files; /* number of files removed in cleanup */
 	size_t	 del_dirs; /* number of directories removed in cleanup */
 	size_t	 brks; /* number of BGPsec Router Key (BRK) certificates */
-	size_t	 brks_invalids; /* invalid BGPsec certs */
-	char	*talnames;
 	struct timeval	elapsed_time;
 	struct timeval	user_time;
 	struct timeval	system_time;
@@ -392,6 +388,10 @@ struct msgbuf;
 
 /* global variables */
 extern int verbose;
+extern const char *tals[];
+extern const char *taldescs[];
+extern unsigned int talrepocnt[];
+extern size_t talsz;
 
 /* Routines for RPKI entities. */
 
@@ -464,11 +464,6 @@ int		 ip_addr_parse(const ASN1_BIT_STRING *,
 			enum afi, const char *, struct ip_addr *);
 void		 ip_addr_print(const struct ip_addr *, enum afi, char *,
 			size_t);
-void		 ip_addr_buffer(struct ibuf *, const struct ip_addr *);
-void		 ip_addr_range_buffer(struct ibuf *,
-			const struct ip_addr_range *);
-void		 ip_addr_read(struct ibuf *, struct ip_addr *);
-void		 ip_addr_range_read(struct ibuf *, struct ip_addr_range *);
 int		 ip_addr_cmp(const struct ip_addr *, const struct ip_addr *);
 int		 ip_addr_check_overlap(const struct cert_ip *,
 			const char *, const struct cert_ip *, size_t);
@@ -507,8 +502,8 @@ void		 rrdp_save_state(size_t, struct rrdp_session *);
 int		 rrdp_handle_file(size_t, enum publish_type, char *,
 		    char *, size_t, char *, size_t);
 char		*repo_filename(const struct repo *, const char *);
-struct repo	*ta_lookup(struct tal *);
-struct repo	*repo_lookup(const char *, const char *);
+struct repo	*ta_lookup(int, struct tal *);
+struct repo	*repo_lookup(int, const char *, const char *);
 int		 repo_queued(struct repo *, struct entity *);
 void		 repo_cleanup(struct filepath_tree *);
 void		 repo_free(void);
@@ -523,6 +518,8 @@ void		 rrdp_fetch(size_t, const char *, const char *,
 		    struct rrdp_session *);
 void		 rrdp_http_done(size_t, enum http_result, const char *);
 
+int		 repo_next_timeout(int);
+void		 repo_check_timeout(void);
 
 /* Logging (though really used for OpenSSL errors). */
 
@@ -600,49 +597,53 @@ int		 output_json(FILE *, struct vrp_tree *, struct brk_tree *,
 
 void		logx(const char *fmt, ...)
 		    __attribute__((format(printf, 1, 2)));
+time_t		getmonotime(void);
 
 int	mkpath(const char *);
 
 #ifndef RPKI_PATH_TAL_DIR
-#define		RPKI_PATH_TAL_DIR	"/etc/rpki"
+#define RPKI_PATH_TAL_DIR	"/etc/rpki"
 #endif
 
 #ifndef RPKI_PATH_OUT_DIR
-#define		RPKI_PATH_OUT_DIR	"/var/db/rpki-client"
+#define RPKI_PATH_OUT_DIR	"/var/db/rpki-client"
 #endif
 
 #ifndef RPKI_PATH_BASE_DIR
-#define		RPKI_PATH_BASE_DIR	"/var/cache/rpki-client"
+#define RPKI_PATH_BASE_DIR	"/var/cache/rpki-client"
 #endif
 
 #ifndef RPKI_CLIENT_USER
-#define		RPKI_CLIENT_USER	"_rpki-client"
+#define RPKI_CLIENT_USER	"_rpki-client"
 #endif
 
 #ifndef RPKI_RSYNC_CMD
-#define		RPKI_RSYNC_CMD		"openrsync"
+#define RPKI_RSYNC_CMD		"openrsync"
 #endif
 
-/*
- * Maximum number of ip ranges and AS ranges we will accept in
- * any single file
- */
-#define MAX_IP_SIZE	200000
-#define MAX_AS_SIZE	200000
+/* Maximum number of IP and AS ranges accepted in any single file */
+#define MAX_IP_SIZE		200000
+#define MAX_AS_SIZE		200000
 
-/*
- * Maximum URI length we will accept
- */
-#define MAX_URI_LENGTH 2048
+/* Maximum acceptable URI length */
+#define MAX_URI_LENGTH		2048
 
-/*
- * Maximum File Size we will accept
- */
-#define MAX_FILE_SIZE 2000000
+/* Maximum acceptable file size */
+#define MAX_FILE_SIZE		2000000
 
-/*
- * Maximum number of FileAndHash entries per Manifest.
- */
-#define	MAX_MANIFEST_ENTRIES	100000
+/* Maximum number of FileAndHash entries per manifest. */
+#define MAX_MANIFEST_ENTRIES	100000
+
+/* Maximum depth of the RPKI tree. */
+#define MAX_CERT_DEPTH		12
+
+/* Maximum number of concurrent rsync processes. */
+#define MAX_RSYNC_PROCESSES	16
+
+/* Maximum allowd repositories per tal */
+#define MAX_REPO_PER_TAL	1000
+
+/* Timeout for repository synchronisation, in seconds */
+#define MAX_REPO_TIMEOUT	(15 * 60)
 
 #endif /* ! EXTERN_H */
