@@ -1,4 +1,4 @@
-/*	$OpenBSD: output-json.c,v 1.20 2021/10/15 08:48:18 job Exp $ */
+/*	$OpenBSD: output-json.c,v 1.22 2021/11/04 11:32:55 claudio Exp $ */
 /*
  * Copyright (c) 2019 Claudio Jeker <claudio@openbsd.org>
  *
@@ -28,6 +28,7 @@ outputheader_json(FILE *out, struct stats *st)
 	char		hn[NI_MAXHOST], tbuf[26];
 	struct tm	*tp;
 	time_t		t;
+	size_t		i;
 
 	time(&t);
 	setenv("TZ", "UTC", 1);
@@ -47,12 +48,27 @@ outputheader_json(FILE *out, struct stats *st)
 	    "\t\t\"failedroas\": %zu,\n"
 	    "\t\t\"invalidroas\": %zu,\n"
 	    "\t\t\"bgpsec_pubkeys\": %zu,\n"
-	    "\t\t\"invalidbgpsec_pubkeys\": %zu,\n"
 	    "\t\t\"certificates\": %zu,\n"
-	    "\t\t\"failcertificates\": %zu,\n"
 	    "\t\t\"invalidcertificates\": %zu,\n"
 	    "\t\t\"tals\": %zu,\n"
-	    "\t\t\"talfiles\": \"%s\",\n"
+	    "\t\t\"invalidtals\": %zu,\n"
+	    "\t\t\"talfiles\": [\n",
+	    hn, tbuf, (long long)st->elapsed_time.tv_sec,
+	    (long long)st->user_time.tv_sec, (long long)st->system_time.tv_sec,
+	    st->roas, st->roas_fail, st->roas_invalid,
+	    st->brks, st->certs, st->certs_fail,
+	    st->tals, talsz - st->tals) < 0)
+		return -1;
+
+	for (i = 0; i < talsz; i++) {
+		if (fprintf(out,
+		    "\t\t\t\"%s\"%s\n",
+		    tals[i], i == talsz - 1 ? "" : ",") < 0)
+			return -1;
+	}
+
+	if (fprintf(out,
+	    "\t\t],\n"
 	    "\t\t\"manifests\": %zu,\n"
 	    "\t\t\"failedmanifests\": %zu,\n"
 	    "\t\t\"stalemanifests\": %zu,\n"
@@ -64,12 +80,6 @@ outputheader_json(FILE *out, struct stats *st)
 	    "\t\t\"cachedir_del_files\": %zu,\n"
 	    "\t\t\"cachedir_del_dirs\": %zu\n"
 	    "\t},\n\n",
-	    hn, tbuf, (long long)st->elapsed_time.tv_sec,
-	    (long long)st->user_time.tv_sec, (long long)st->system_time.tv_sec,
-	    st->roas, st->roas_fail, st->roas_invalid,
-	    st->brks, st->brks_invalids,
-	    st->certs, st->certs_fail, st->certs_invalid,
-	    st->tals, st->talnames,
 	    st->mfts, st->mfts_fail, st->mfts_stale,
 	    st->crls,
 	    st->gbrs,
@@ -106,7 +116,8 @@ output_json(FILE *out, struct vrp_tree *vrps, struct brk_tree *brks,
 
 		if (fprintf(out, "\t\t{ \"asn\": %u, \"prefix\": \"%s\", "
 		    "\"maxLength\": %u, \"ta\": \"%s\", \"expires\": %lld }",
-		    v->asid, buf, v->maxlength, v->tal, (long long)v->expires)
+		    v->asid, buf, v->maxlength, taldescs[v->talid],
+		    (long long)v->expires)
 		    < 0)
 			return -1;
 	}
@@ -124,7 +135,7 @@ output_json(FILE *out, struct vrp_tree *vrps, struct brk_tree *brks,
 
 		if (fprintf(out, "\t\t{ \"asn\": %u, \"ski\": \"%s\", "
 		    "\"pubkey\": \"%s\", \"ta\": \"%s\", \"expires\": %lld }",
-		    b->asid, b->ski, b->pubkey, b->tal,
+		    b->asid, b->ski, b->pubkey, taldescs[b->talid],
 		    (long long)b->expires) < 0)
 			return -1;
 	}
